
--Copyright (c) 1993 Ohio Board of Regents and the University of
--Cincinnati.  All Rights Reserved.


--example in the construction of petri nets using the vhdl design
--entities and package of in this directory.  this example
--instantiates the following petri net.


--    P1(3)      P2(5)
--    (place)    (place)
--      |          |
--      |          +-------+--+
--      |          |       |  |
--      +-------------+    |  |
--      |          |  |    |  |
--      |          |  |    |  |
--      |  +------ +  |    |  |
--      |  |       |  |    |  |
--      |  |       |  |    |  |
--      |  |       |  |    |  |
--      |  |       |  |    |  |
--     ------     ------   |  |
--       T1         T2     |  |
--     (trans)    (trans)  |  |
--        |          |     |  |
--        |          |     |  |
--        |          |     |  |
--        |  +-------+     |  |
--        |  |             |  |
--        |  |             |  |
--        |  |             |  |
--        P3(0)            |  |
--        (place)          |  |
--          |              |  |
--          |              |  |
--          +-----------+  |  |
--                      |  |  |
--                      |  |  |
--                      |  |  |
--                      |  |  |
--                     ---------
--                       T3
--                     (trans)
--                        |
--                        |
--                        |
--                       P4(0)
--                       (place)


library petri;
use petri.petri.all;
use petri.math.all;

entity petri_use is

end;


architecture demo of petri_use is

signal arc1, arc2, arc3, arc4, arc5, 
       arc6, arc7, arc8, arc9, arc10 : arc bus; 


component place_0_2

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (out_arc1, out_arc2 : inout arc bus );

end component;


component place_0_4

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (out_arc1, out_arc2, out_arc3, out_arc4 : inout arc bus);
    
end component;

component place_1_0

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (in_arc1            : inout arc bus);

end component;


component place_2_1

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (in_arc1,  in_arc2  : inout arc bus;
          out_arc1           : inout arc bus);

end component;

component transition_2_1

    generic (transition_name : string;
	     petri_typ       : p_type := live;
             debug_cntrl     : debug := firing);

    port (in_arc1,  in_arc2  : inout arc bus;
          out_arc1           : inout arc bus);

end component;

component transition_3_1

    generic (transition_name : string;
	     petri_typ       : p_type := live;
             debug_cntrl     : debug := firing);

    port (in_arc1,  in_arc2,  in_arc3  : inout arc bus;
          out_arc1                     : inout arc bus);

end component;


for P1 : place_0_2 use entity work.place_0_2(structure);
for P2 : place_0_4 use entity work.place_0_4(structure);
for P3 : place_2_1 use entity work.place_2_1(structure);
for P4 : place_1_0 use entity work.place_1_0(structure);

for T1, T2 : transition_2_1 use entity work.transition_2_1(structure);
for T3     : transition_3_1 use entity work.transition_3_1(structure);

begin

P1 : place_0_2 
     generic map (32, "P1")
     port map (arc1, arc2);
P2 : place_0_4
     generic map (51, "P2")
     port map (arc3, arc4, arc5, arc10);
T1 : transition_2_1
     generic map ("T1")
     port map (arc1, arc3, arc6);
T2 : transition_2_1
     generic map ("T2")
     port map (arc2, arc4, arc7);
P3 : place_2_1
     generic map (0, "P3")
     port map (arc6, arc7, arc8);
T3 : transition_3_1
     generic map ("T3")
     port map (arc8, arc5, arc10, arc9);
P4 : place_1_0
     generic map (0, "P4")
     port map (arc9);

end;
