
--Copyright (c) 1993 Ohio Board of Regents and the University of
--Cincinnati.  All Rights Reserved.


--example in the construction of petri nets using the vhdl design
--entities and package of in this directory.  this example
--instantiates the following petri net.


--    P1(3)  
--    (place)
--      |    
--      | 
--    ------  
--      T1    
--    (trans) 
--      |          P5
--      |        (place) 
--     P2(0)       | |
--    (place)      | |
--      |  +-------+ |
--      |  |	     |
--     ------	     |
--       T2	     |
--     (trans)	     |	
--       |	     |
--       |	     |
--      P3(0)	     |
--     (place)	     |
--       |  ------+  |	
--       |  	  |  |
--     -----	------ 
--	T3	 T4
--    (trans)    
--       |
--       |
--      P4
--     (place)

library petri;
use petri.petri.all;
entity petri_use is

end;


architecture demo of petri_use is

signal arc1, arc2, arc3, arc4, arc5, 
       arc6, arc7, arc8, arc9, arc10 : arc bus; 

disconnect all : arc after 0 ns;

component place_0_1

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (out_arc1 : inout arc bus );

end component;

component place_1_1

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (in_arc1, out_arc1 : inout arc bus );

end component;


component place_1_2

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (in_arc1, out_arc1, out_arc2 : inout arc bus);
    
end component;

component place_1_0

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (in_arc1            : inout arc bus);

end component;


component place_2_1

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (in_arc1,  in_arc2  : inout arc bus;
          out_arc1           : inout arc bus);

end component;

component transition_1_1

    generic (transition_name : string;
             debug_cntrl     : debug := firing);

    port (in_arc1	     : inout arc bus;
          out_arc1           : inout arc bus);

end component;

component transition_2_1

    generic (transition_name : string;
             debug_cntrl     : debug := firing);

    port (in_arc1,  in_arc2  : inout arc bus;
          out_arc1           : inout arc bus);

end component;

component transition_1_2

    generic (transition_name : string;
             debug_cntrl     : debug := firing);

    port (in_arc1            : inout arc bus;
          out_arc1, out_arc2 : inout arc bus);

end component;


for P1 : place_0_1 use entity work.place_0_1(structure);
for P2 : place_1_1 use entity work.place_1_1(structure);
for P3 : place_1_2 use entity work.place_1_2(structure);
for P4 : place_1_0 use entity work.place_1_0(structure);
for P5 : place_1_1 use entity work.place_1_1(structure);

for T1, T4 : transition_1_1 use entity work.transition_1_1(structure);
for T2     : transition_2_1 use entity work.transition_2_1(structure);
for T3     : transition_1_2 use entity work.transition_1_2(structure);

begin

P1 : place_0_1 
     generic map (10, "P1")
     port map (arc1);
P2 : place_1_1
     generic map (0, "P2")
     port map (arc1, arc2);
T1 : transition_1_1
     generic map ("T1")
     port map (arc1, arc2);
T2 : transition_2_1
     generic map ("T2")
     port map (arc3, arc10, arc4);
T3 : transition_1_2
     generic map ("T3")
     port map (arc5, arc6, arc7);
P3 : place_1_2
     generic map (0, "P3")
     port map (arc4, arc5, arc8);
T4 : transition_1_1
     generic map ("T4")
     port map (arc8, arc9);
P4 : place_1_0
     generic map (0, "P4")
     port map (arc6);
P5 : place_1_1
     generic map (0, "P5")
     port map (arc9, arc10);
end;
