
--Copyright (c) 1993 Ohio Board of Regents and the University of
--Cincinnati.  All Rights Reserved.

--author: sidhartha mohanty, philip a. wilsey.
--smohanty@thor.ece.uc.edu, phil.wilsey@uc.edu


library petri;
use petri.petri.all;
use petri.math.all;
use std.textio.all;
entity transition is

    generic (number_in_arcs  : integer := 1;
             number_out_arcs : integer := 1;
             trans_name      : string  := "";
	     petri_type      : p_type  := live;
             debug_control   : debug   := none);

    port (in_arc  : inout arc_array (1 to number_in_arcs) bus;
          out_arc : inout arc_array (1 to number_out_arcs) bus);
    
end;


architecture behavior of transition is


--unique identifier read from file produced by disambiguate is stored
--in identifer.  

    signal identifier : integer := 1;
    signal identifier_in_fs : time := 1 fs;


--in_arc_list and out_arc_list store the identifier numbers from the
--places in, respectively, the input arcs and the output arcs.

    signal in_arc_list  : arc_list (1 to number_in_arcs);
    signal out_arc_list : arc_list (1 to number_out_arcs);

--used to convert transition identifier from integer to time.  the
--time value equivalent of the identifier is used to move the
--transition to unique minor cycle for evaluation of firing
--capability.

function convert_integer_to_time (number : integer) return time is

variable wk_time : time := 1 fs;
    
begin
    wk_time := 0 fs;
    for i in 1 to number loop
    	wk_time := wk_time + 1 fs;
    end loop;

    return wk_time;

end convert_integer_to_time;

function rand_integer_to_time (number : integer) return time is

variable wk_time : time := 1 fs;
variable rand1 : integer := 0;
variable rand2 : real := 0.0;
variable rand3 : integer := 0;

begin

    wk_time := 0 fs;
    ran(2,number,10.0,45.0,rand1,rand2);
    rand3 := integer(rand2 * 10.0);
    wk_time := (rand3 * 1 fs);

    return wk_time;

end rand_integer_to_time;

begin


--get disambiguating identifier from file produced by disambiguate.

initialize: process 

    variable integer_line : line;
    variable integer_id : integer := 1;

begin
    for i in 1 to number_out_arcs loop
	out_arc(i) <= null;
    end loop;

    for i in 1 to number_in_arcs loop
	in_arc(i) <= null;
    end loop;

    wait for 0 fs;

    assert not std.textio.endfile(std.textio.input)
        report "(transition.vhdl) Number of places and transitions exceed max_places_and_transitions.  Increase the value in package petri."
        severity failure;

    readline(std.textio.input, integer_line);

    assert integer_line'length >= 0 
        report "(transition.vhdl) Missing integer in file used to disambiguate places and transitions.  Check to make certain that disambiguate was run properly."
        severity failure;

    read(integer_line, integer_id);
    identifier <= integer_id;
--    identifier_in_fs <= convert_integer_to_time(integer_id);

    wait for 0 fs;
    
    if debug_control = handshaking or debug_control = everything then 
    	print_debug(id_a_transition, debug_control, 
	            trans_name, identifier,
                    "Waiting for identify handshakes.");
    end if;

    wait on in_arc, out_arc;

    for i in 1 to number_in_arcs loop 

      if in_arc(i).handshake = identify_place then

    	    if debug_control = handshaking or
	       debug_control = everything then 
   	    	print_debug(id_a_transition, debug_control, 
	            	    trans_name, integer_id,
                    	    "Received identify handshake from " &
			    stringof(in_arc(i).number) & ".");
	    end if;

	    in_arc_list(i) <= in_arc(i).number;
	    wait for 0 fs;

	else

--    	    print_debug(id_a_transition, debug_control, 
--	            	trans_name, integer_id,
--                    	"My input ports must all be connected to places.");

--	    assert false severity failure;

    	end if;

    end loop;

    for i in 1 to number_out_arcs loop

    	if out_arc(i).handshake = identify_place then

    	    if debug_control = handshaking or
	       debug_control = everything then 
    	    	print_debug(id_a_transition, debug_control, 
	            	    trans_name, integer_id,
                    	    "Received identify handshake from " &
	                    stringof(out_arc(i).number) & ".");
	    end if;

	    out_arc_list(i) <= out_arc(i).number;
            wait for 0 fs;

	else

--    	    print_debug(id_a_transition, debug_control, 
--	            	trans_name, integer_id,
--                  	"My output ports must all be connected to places.");

--	    assert false severity failure;

    	end if;

    end loop;

    for i in 1 to number_in_arcs loop
    	in_arc(i) <= null;
    end loop;

    for i in 1 to number_out_arcs loop
    	out_arc(i) <= null;
    end loop;

    wait for 1 fs;

    wait;  --end place identification phase

end process;


--identify self to places connected to my ports

identify: process

    variable wk_time : time := 1 fs;

begin

    wait for 1 fs;
    wk_time := identifier_in_fs - 1 fs;
    wait for wk_time;
    
    if (debug_control = handshaking or
        debug_control = everything) then 
    	print_debug(id_a_transition, debug_control, 
	            trans_name, identifier,
                    "Identifying self to places.");
    end if;

    for i in 1 to number_in_arcs loop
	in_arc(i) <= (identify_transition, identifier);
    end loop;
    
    for i in 1 to number_out_arcs loop
	out_arc(i) <= (identify_transition, identifier);
    end loop;
    
    wait for 0 fs; 

    if (debug_control = handshaking or
        debug_control = everything) then 
    	print_debug(id_a_transition, debug_control,
	            trans_name, identifier,
                    "Ending identification phase.");
    end if;

    for i in 1 to number_in_arcs loop
    	in_arc(i) <= null;
    end loop;

    for i in 1 to number_out_arcs loop
    	out_arc(i) <= null;
    end loop;
    
    wait for 0 fs;

    wait;  --end transition identification phase

end process;


--use identifier to move to unique time (minor cycle) and determine if
--transition can fire.  tickle input places for tokens and fire if all
--present;  otherwise wait until next major cycle.

run: process 

    variable wk_handshake : token_handshake;
    variable number_tokens_present : integer := 1;
    variable avail : bool_array(1 to number_in_arcs);
    variable rand1 : integer := 0;
    variable rand2 : real := 1.0; 
    variable rand3 : integer := 0;
    variable I : integer := 0;

begin
    if debug_control = everything then 		   
        print_debug(id_a_transition, debug_control, 
	            trans_name, identifier,
                    "Advancing run process past identification phase.");
    end if;

    wait for 1 ns;  --delay for initializing place and transition identifiers.

    if debug_control = everything then 		   
        print_debug(id_a_transition, debug_control, 
	            trans_name, identifier,
                    "Moving to first major cycle boundary.");
    end if;

    if (petri_type = live) then
       identifier_in_fs <= convert_integer_to_time(identifier);
    else 
      if (I < 1) then
         rand3 := identifier;
         I := I + 1;
      end if;
      ran(2, rand3, 10.0, 30.0, rand1, rand2);
      identifier_in_fs <= rand_integer_to_time(rand3);
      rand3 := integer(rand2 * 10.0);
      wait for 0 fs;
    end if;

    wait for identifier_in_fs;  -- move to unique minor cycle for firing.

    if debug_control = everything then 		   
        print_debug(id_a_transition, debug_control, 
	            trans_name, identifier,
                    "Arrived at first major cycle boundary.");
    end if;

    loop

    	if debug_control = handshaking or debug_control = everything then 
    	    print_debug(id_a_transition, debug_control, 
	            trans_name, identifier,
	                "Querying place(s) for input token information.");
	end if;

    	for i in 1 to number_in_arcs loop
	    in_arc(i) <= (send, identifier);
    	end loop;

	wait for 0 fs;

     	if debug_control = handshaking or debug_control = everything then 
    	    print_debug(id_a_transition, debug_control, 
	            trans_name, identifier,
	                "Waiting for token information....");
	end if;

        for i in 1 to number_in_arcs loop
	 if in_arc(i).handshake = send then
	    in_arc(i) <= null;
         end if;
    	end loop;

        wait for 0 fs;

	wait on in_arc for 1 ns;

    	if debug_control = handshaking or debug_control = everything then 
   	    print_debug(id_a_transition, debug_control, 
	            trans_name, identifier,
	                "Received token information.");
	end if;

    	wk_handshake := present;
	number_tokens_present := 0;

    	for i in 1 to number_in_arcs loop

    	    if in_arc(i).handshake /= present then
	    	wk_handshake := not_driving;
	    else
	    	number_tokens_present := number_tokens_present + 1;
	    end if;

    	end loop;

	if (wk_handshake = present) then

    	    if debug_control = handshaking or
	       debug_control = everything then 		   
    	    	print_debug(id_a_transition, debug_control, 
	                    trans_name, identifier,
	                    "Consuming input tokens.");
	    end if;

   	    if debug_control = firing or
	       debug_control = everything then 		   
    	        print_debug(id_a_transition, debug_control, 
	                    trans_name, identifier,
	                    "Firing....");
	    end if;

    	    for i in 1 to number_in_arcs loop
	      if in_arc(i).handshake = present then
		avail(i) := TRUE;
	      end if;
	    end loop;

	    for i in 1 to number_in_arcs loop
		in_arc(i) <= null;
            end loop;    

            wait for 0 fs;

	    for i in 1 to number_in_arcs loop
	      if avail(i) = TRUE then
		avail(i) := FALSE;
	    	in_arc(i) <= (firing, identifier);
	      end if;
	    end loop;

	    wait for 0 fs;

            for i in 1 to number_in_arcs loop
	      if in_arc(i).handshake = firing then
                in_arc(i) <= null;
	      end if;
	    end loop;

            wait for 0 fs;

    	    for i in 1 to number_out_arcs loop
		out_arc(i) <= (sending, identifier);
	    end loop;

    	    if debug_control = handshaking or
	       debug_control = everything then 		   
    	    	print_debug(id_a_transition, debug_control, 
	                    trans_name, identifier,
	                    "Sending token to each output place.");
	    end if;

	    wait for 0 fs;

    	    for i in 1 to number_in_arcs loop
	    	in_arc(i) <= null;
	    end loop;

    	    for i in 1 to number_out_arcs loop
		out_arc(i) <= null;
	    end loop;

	    wait for 0 fs;

    	    if debug_control = handshaking or
	       debug_control = everything then 		   
    	    	print_debug(id_a_transition, debug_control, 
	                    trans_name, identifier,
	                    "Finished consuming input tokens.");
	    end if;

 
    	    if debug_control = handshaking or
	       debug_control = everything then 		   
    	    	print_debug(id_a_transition, debug_control, 
	                    trans_name, identifier,
	                    "Finished broadcasting output token.");
	    end if;

 
    	else

    	    if debug_control = handshaking or
	       debug_control = everything then 
    	    	print_debug(id_a_transition, debug_control, 
	                    trans_name, identifier,
	                    "Need tokens from " &
			    stringof(number_in_arcs) &
			    " places.  Received token present handshake " &
			    stringof(number_tokens_present) & " places.");
	   end if;

	end if;
	
    	if debug_control = everything then 		   
    	    print_debug(id_a_transition, debug_control, 
	                trans_name, identifier,
                        "Moving to next major cycle.");
    	end if;

	wait for 1 ns;  --move to the middle of the next major cycle.

    end loop;

end process;

end behavior;
